/* Copyright and licensing - see the license block at the start of multiscrollOverlay.xul */

// mergeObjects(base, [overlay, ... ]) - does what it says: overlays overlay, ... onto  
// base and returns the resulting object
// Note that this will DESTROY getters in overlay unless protected by an object
// wrapper not in base at that point in the process.
// Instead of mergeObjects(base, { get attr() { ... } }); use base.attr getter= { ... };
function mergeObjects(base) {
	if (!base instanceof Object)
		return base;
	for (var i = 1; i < arguments.length; i++)
		if (arguments[i] instanceof Object)
			for (var j in arguments[i]) {
				if (base[j] == undefined || base[j] == null)
					base[j] = arguments[i][j];
				else if (base[j] instanceof Object && arguments[i][j] instanceof Object)
					mergeObjects(base[j], arguments[i][j]);
			}
	return base;
}

var multiscroll = {
	scroll : {}, 
	pan : {
		wireframe : 0, textured : 1, thumbnail : 2
	},
	drag : {}, 
	mouse : {
		button : {
			left : 0, middle : 1, right : 2,
			action : function(button) {
				return [multiscroll.action.left, multiscroll.action.middle, multiscroll.action.right][button];
			}
		},
		action : {
			mousedown : 0, click : 1, doubleclick : 2, release : 3, drag : 4
		}
	},
};
mergeObjects(multiscroll, {
	documentWidth : null,
	documentHeight : null,
	windowWidth : null,
	windowHeight : null,
	clientWnd : null,
	clientFrame : null,
	clientFrameDoc : null,
	clientFrameBody : null,
	startX : null,
	startY : null,
	clientX : null,
	clientY : null,
	savedX : null,
	savedY : null,
	browserClientX : null,
	browserClientY : null,
	snapOn : null,
});
multiscroll.tool = {
		scroll : multiscroll.scroll,
		pan : multiscroll.pan,
		drag : multiscroll.drag,
		get active() {
			return multiscroll.scroll.active ? multiscroll.scroll
				: multiscroll.pan.active ? multiscroll.pan
				: multiscroll.drag.active ? multiscroll.drag
				: multiscroll.none;
		},
		byCode : function(value) {
			return value == 0 ? multiscroll.scroll
				: value == 3 ? multiscroll.pan
				: value == 4 ? multiscroll.drag
				: multiscroll.none;
		},
		code : function(tool) {
			return tool == multiscroll.scroll ? 0
				: tool == multiscroll.pan ? 3
				: tool == multiscroll.drag ? 4
				: -1;
		}
	};
mergeObjects(multiscroll, {
	scroll : {
		settings : {
			multiplier : 0.2,
			exponent : 2,
		},
		active : false,
		done : false,
	},
	pan : {
		settings : {
			size : 100,
			appearance : multiscroll.pan.wireframe,
		},
		active : false,
		done : false,
	},
	drag : {
		settings : {
			multiplier : 1,
			exponent : 2,
		},
		active : false,
		done : false,
	}
});
multiscroll.none = {
		get active() { 
			return !multiscroll.scroll.active && !multiscroll.pan.active && !multiscroll.drag.active;
		},
		set active(value) {
			return value ? !(multiscroll.scroll.active = multiscroll.pan.active = multiscroll.drag.active = false)
				: multiscroll.none.active ? !(multiscroll.action.Default.tool.active = true)
				: value;
		},
		get done() { 
			return !multiscroll.scroll.done && !multiscroll.pan.done && !multiscroll.drag.done;
		},
		start : function(evt) {
			return multiscroll.scroll.active && multiscroll.scroll.stop(evt), 
				multiscroll.pan.active && multiscroll.pan.stop(evt), 
				multiscroll.drag.active && multiscroll.drag.stop(evt);
		},
		stop : function() { return null; },
		Do : function() { return null; }
	};
multiscroll.action = {
		Default : {
			start : function(evt) {
				multiscroll.snapOn = true;
				multiscroll.clientFrame = evt.originalTarget.ownerDocument.defaultView;
				multiscroll.clientFrameDoc = evt.originalTarget.ownerDocument;
				multiscroll.clientFrameBody = (multiscroll.clientFrameDoc.getElementsByTagName('body')[0])
								 ? multiscroll.clientFrameDoc.getElementsByTagName('body')[0]
								 : multiscroll.clientFrameDoc.getElementsByTagName('*')[0];
				multiscroll.clientWnd = evt.originalTarget.ownerDocument.contentWindow;
				multiscroll.savedX = multiscroll.clientFrame.pageXOffset;
				multiscroll.savedY = multiscroll.clientFrame.pageYOffset;

				multiscroll.action.Default.tool.settings = multiscroll.action.Default.settings;
				multiscroll.action.Default.tool.start(evt);
			},
			stop : function(evt) {
				multiscroll.action.Default.tool.stop(evt);
			},
			get active() {
				return multiscroll.action.Default.tool.active;
			},
			set active(value) {
				return multiscroll.action.Default.tool.active = value;
			},
			get done() {
				return multiscroll.action.Default.tool.done;
			},
		},
		left : {
			start : function(evt) {
				multiscroll.action.left.tool.settings = multiscroll.action.left.settings;
				multiscroll.action.left.tool.start(evt);
			},
			stop : function(evt) {
				multiscroll.action.left.tool.stop(evt);
			},
			get active() {
				return multiscroll.action.left.tool.active;
			},
			set active(value) {
				return multiscroll.action.left.tool.active = value;
			},
			get done() {
				return multiscroll.action.left.tool.done;
			},
		},
		middle : {
			start : function(evt) {
				multiscroll.action.middle.tool.settings = multiscroll.action.middle.settings;
				multiscroll.action.middle.tool.start(evt);
			},
			stop : function(evt) {
				multiscroll.action.middle.tool.stop(evt);
			},
			get active() {
				return multiscroll.action.middle.tool.active;
			},
			set active(value) {
				return multiscroll.action.middle.tool.active = value;
			},
			get done() {
				return multiscroll.action.middle.tool.done;
			},
		},
		right : {
			start : function(evt) {
				multiscroll.action.right.tool.settings = multiscroll.action.right.settings;
				multiscroll.action.right.tool.start(evt);
			},
			stop : function(evt) {
				multiscroll.action.right.tool.stop(evt);
			},
			get active() {
				return multiscroll.action.right.tool.active;
			},
			set active(value) {
				return multiscroll.action.right.tool.active = value;
			},
			get done() {
				return multiscroll.action.right.tool.done;
			},
		}
	};
mergeObjects(multiscroll, {
	isLink : function(node) {
		if (!node)
			return false;
		else if (node.nodeName == "A" || node.nodeName == "INPUT"
			 || node.nodeName == "TEXTAREA" || node.nodeName == "AREA")
			return true;
		else
			return this.isLink(node.parentNode);
	},
	isChrome : function(target) {
	        // Is there a better way to do this?
	        if (target.id != "content")
	                return true;
	        else
	                return false;
	},
	isTabStrip : function(target) {
		// There must be a better way!
		if (target.nodeName == "tab" || target.nodeName.slice(0, 4) == "xul:" )
			return true;
		else
			return false;
	}
});